// tree.js -- code to support the index tree (in archangel.xul).
// $Header: /cvs/archangel/src/archangel/content/tree.js,v 1.4 2003/02/03 06:33:05 marnanel Exp $
//
// Copyright (c) 2002 Thomas Thurman
// thomas@thurman.org.uk
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have be able to view the GNU General Public License at 
// http://www.gnu.org/copyleft/gpl.html ; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

////////////////////////////////////////////////////////////////

var server = 0;   // RGTP connection to the server
var vault = 0;    // RDF file containing all our state
var groggs = 0;   // Our (currently) only server, inside "serverlist"
var backing_store = 0;

var known_sequences = 0;

function createVault() {
	// FIXME: Need good error checking all the way through.

	var du = new DirUtils();

	vault = new RDFFile(new Dir(du.getMozUserHomeDir()).append("rgtpserv.rdf"));

	if (vault==null) throw 'problem: vault is null';

	// GROGGS (server s1) is hard-coded in this release.

	groggs = vault.addRootBag("urn:servers:s1");

	if (!groggs.getAttribute('host')) {
		groggs.setAttribute('host', 'rgtp-serv.groggs.group.cam.ac.uk');
		groggs.setAttribute('port', 1431);
	}

	vault.flush();
}

function connectToServer() {
	server = new RGTPConnection(groggs);
	server.open();
}

function prettyDate(date) {
	return new Date(date*1000).toGMTString();
}

function populateTree() {
	var items = vault.getRootBag('urn:servers:s1').getSubNodes();

	known_sequences = [];
	details = [];

	for (var i=0; i<items.length; i++) {
		var itemid = items[i].getSubject();
		itemid = itemid.substring(itemid.length-8);

		var known = items[i].getAttribute('known');

		known_sequences.push(known);

		details[known] = ['',
			itemid,
			items[i].getAttribute('subject'),
			prettyDate(items[i].getAttribute('last-date')),
			items[i].getAttribute('last-by'),
			items[i].getAttribute('replies'),
		];
	}

	known_sequences.sort();
	known_sequences.reverse();

	for (var j=0; j<known_sequences.length; j++) {
		addToIndex(details[known_sequences[j]]);
	}

}

function catcher(code) {
	try {
		eval(code);
	} catch(e) {
		alert(code+': '+e);
		throw e;
	}
}

function treeStart() {
	createVault();
	connectToServer();
}

var inItemHead = 0;
var itemHeaderBlock = [];
var grogname = 0;
var userid = 0;

var shortHeadForm = /from (.*) \(([^(]*)\) at/;
var longHeadFormUserid = /from ([^ ]*) at/;
var longHeadFormGrogname = /^From (.*)$/;
var initialItemLine = 0;

function indexClick() {
	var line = document.getElementById('index-tree').currentIndex;

	if (line==0) return;
	line = line - 1; // ick

	var itemid = document.getElementById('groggs-items').childNodes[line].firstChild.childNodes[1].getAttribute('label');

	clearItemLines();
	inItemHead = 0;
	initialItemLine = 1;
	server.getItem(itemid);
}

function receivedItemLine(line) {

	if (initialItemLine) {
		// should process it somehow, but for now...
		initialItemLine = 0;
		return;
	}

	if (!inItemHead && line[0]=='^') {
		// Start of an item head
		inItemHead = 1;
		grogname = 'Grogname?';
		userid = 'Userid?';
		itemHeaderBlock = [prettyDate('0x'+line.substring(10))];
	} else if (inItemHead && line=='') {
		itemHeaderBlock.push(userid);
		itemHeaderBlock.push(grogname);
		addAttributionBlock(itemHeaderBlock);
		inItemHead = 0;
	} else if (inItemHead) {
			if (shortHeadForm.test(line)) {
				grogname = RegExp.$1;
				userid = RegExp.$2;
		    } else if (longHeadFormGrogname.test(line))
				grogname = RegExp.$1;
		    else if (longHeadFormUserid.test(line))
				userid = RegExp.$1;
	} else
		addItemLine(line);
}
